import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class MlpVisual extends JFrame implements ActionListener, MouseListener {
  Vector inData;
  PaintBoard board;
  JButton reset, red, green;
  Color dotColor;
  MLP mainWindow;

  public MlpVisual(MLP mlp){
    inData = new Vector();
    dotColor = Color.black;

    mainWindow = mlp;
    getContentPane().setLayout(new BorderLayout());

    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(1,3));
    reset = new JButton("Reset");
    reset.addActionListener(this);
    buttonPanel.add(reset);
    red = new JButton("Red");
    red.addActionListener(this);
    buttonPanel.add(red);
    green = new JButton("Green");
    green.addActionListener(this);
    buttonPanel.add(green);
    this.getContentPane().add(buttonPanel, BorderLayout.NORTH);

    board = new PaintBoard();
    board.addMouseListener(this);
    this.getContentPane().add(board, BorderLayout.CENTER);


    this.setSize(350,350);
    this.setVisible(true);
  }

  public void setInData(Vector in) {
    inData = in;
    repaint();
  }

  private class PaintBoard extends JPanel{

    private PaintBoard() {
      this.setBackground(Color.black);
    }

    public Dimension getMinimumSize() {
      return new Dimension(350, 350);
    }

    public Dimension getPreferredSize() {
      return getMinimumSize();
    }


    public void paintComponent(Graphics g) {
      g.setColor(Color.black);
      super.paintComponent(g);

      int sizeX = this.getWidth();
      int sizeY = this.getHeight();
      int maxKoordX = 1 * sizeX -80;
      int maxKoordY = 1 * sizeY -80;
      int minKoord = 30;

      //draw Koordinate Axis
      g.setColor(Color.gray);
      g.fillRect(minKoord+5, minKoord-10, 2, maxKoordY+minKoord);
      g.fillRect(minKoord-7, maxKoordY+minKoord+5, maxKoordX+minKoord,2);

      g.fillRect(minKoord-2, minKoord+5, 10, 2);
      g.drawString("1", minKoord-10, minKoord+10);

      g.drawString("0", minKoord-10, maxKoordY+minKoord+20);

      g.fillRect(maxKoordX+minKoord+5, maxKoordY+minKoord+5,2,10);
      g.drawString("1",maxKoordX+minKoord-10,maxKoordY+minKoord+20);

      for (int i = 0; i < inData.size(); i++) {
          double[] koord = (double[]) inData.get(i);
          if (koord[2] > 0.5)
            g.setColor(Color.green);
          else
            g.setColor(Color.red);
          g.fillOval( (int) ( (koord[0] * maxKoordX)+minKoord),
                     (int) ( maxKoordY-(koord[1] * maxKoordY)+minKoord), 10, 10);
      }
      for (int i=minKoord+5; i<=maxKoordY+minKoord+5; i+=5) {
        for (int j=minKoord+5; j<=maxKoordX+minKoord+5; j+=5) {
          ((Neuron)mainWindow.input.get(0)).setOutput((j-minKoord)/(double)maxKoordX);
          ((Neuron)mainWindow.input.get(1)).setOutput((i-minKoord)/(double)maxKoordY);

          mainWindow.algo.doFeedForward(mainWindow.input, mainWindow.hidden, mainWindow.output);


          if (((Neuron)mainWindow.output.get(0)).getOutput()>0.5) g.setColor(Color.green);
            else g.setColor(Color.red);

          g.fillOval(j,maxKoordY-i+70,2,2);
        }
      }
    }
  }

  public void actionPerformed(ActionEvent evt) {
    Object source = evt.getSource();

    if (source == green) {
      dotColor = Color.green;
    }
    else if (source == red) {
      dotColor = Color.red;
    }
    else if (source == reset) {
      mainWindow.resetMlpDisplay();
    }
  }

  public void mouseClicked(MouseEvent evt) {}

  public void mousePressed(MouseEvent evt) {
    Object target = evt.getSource();
    if (target == board) {
      int sizeX = board.getWidth();
      int sizeY = board.getHeight();
      int maxKoordX = 1 * sizeX -80;
      int maxKoordY = 1 * sizeY -80;
      int minKoord = 30;

      double xCoord = ((double)(evt.getX()-5)-minKoord)/maxKoordX;
      double yCoord = ((double)(evt.getY()-5)-minKoord-maxKoordY)/(-maxKoordY);

      if (dotColor==Color.red) {
        mainWindow.addTrainData(xCoord, yCoord, 0.0);
      }
      else if (dotColor == Color.green) {
        mainWindow.addTrainData(xCoord, yCoord, 1.0);
      }

    }

  }
  public void mouseReleased(MouseEvent evt){}
  public void mouseEntered(MouseEvent evt) {}
  public void mouseExited(MouseEvent evt) {}


}
