import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.Math;
import java.io.*;
import java.net.*;

public class Kohonen extends JApplet implements AdjustmentListener, ActionListener {
    JButton reset, train, test, succ, pred;
    JTextArea lessons;
    JScrollBar nNeurons, mNeurons;
    JPanel mainframe;
    int nNeur, mNeur, invec, iteration=0;
    JLabel nNeuron, mNeuron, inputVec, rad, rat, itnr;
    KohonenFrame kFrame;
    Neuron[][] neuron;
    KohonenAlgo algo;
    InputVectorChoice choose;
    double radius = 100, rate=1.0;
    Vector input, lineInput, stimmax, oldStim;
    KohonenThreadTrain training;
    KohonenThreadTest testing;
    boolean tested;
    JMenuItem openFile;
   
    public Kohonen(){
	algo = new KohonenAlgo();
	tested = false;
	input = new Vector();
	stimmax = new Vector();

	JMenuBar menu = new JMenuBar();
	JMenu file = new JMenu("File");
	openFile = new JMenuItem("Open File");
	openFile.addActionListener(this);
	file.add(openFile);
	menu.add(file);
	
	setJMenuBar(menu);
	
	mainframe = new JPanel();
	mainframe.setLayout(new BorderLayout(0,10));

	JPanel north = new JPanel();
	north.setLayout(new GridLayout(1,6));

	reset = new JButton("Reset");
	reset.addActionListener(this);
	north.add(reset);

	train = new JButton("Start Training");
	train.addActionListener(this);
	north.add(train);
	
	test = new JButton("Start Test");
	test.addActionListener(this);
	north.add(test);

	choose = new InputVectorChoice();

	succ = new JButton("+");
	succ.addActionListener(this);
	north.add(succ);

	inputVec = new JLabel("InputVectors: 5");
	inputVec.setHorizontalAlignment(JLabel.CENTER);
	invec = 5;
	north.add(inputVec);

	pred = new JButton("-");
	pred.addActionListener(this);
	north.add(pred);

	mainframe.add(north, BorderLayout.NORTH);

	JPanel south = new JPanel();
	south.setLayout(new GridLayout(2,2));

        mNeuron = new JLabel("Nr. of Neurons m: 10  ");
	mNeuron.setHorizontalAlignment(JLabel.RIGHT);
	south.add(mNeuron);

       	mNeurons = new JScrollBar(JScrollBar.HORIZONTAL,10,1,5,21);
	mNeurons.addAdjustmentListener(this);
	south.add(mNeurons);
	mNeur = 10;

        nNeuron = new JLabel("Nr. of Neurons  n: 10  ");
	nNeuron.setHorizontalAlignment(JLabel.RIGHT);
	south.add(nNeuron);

	nNeurons = new JScrollBar(JScrollBar.HORIZONTAL,10,1,5,21);
	nNeurons.addAdjustmentListener(this);
	south.add(nNeurons);
	nNeur = 10;
	
	mainframe.add(south, BorderLayout.SOUTH);

	kFrame = new KohonenFrame();
	kFrame.setLayout(null);
	kFrame.setBackground(Color.lightGray);
	mainframe.add(kFrame, BorderLayout.CENTER);

	JPanel west = new JPanel();
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints constraints = new GridBagConstraints();
	west.setLayout(gridbag);

	JPanel box = new JPanel();
	lessons = new JTextArea(16,15);
	lessons.setLineWrap(true);
	box.add(new JScrollPane(lessons));
	buildConstraints(constraints, 0,0,3,1,50,50);
	constraints.fill = GridBagConstraints.BOTH;
	gridbag.setConstraints(box, constraints);
	west.add(box);

	itnr = new JLabel("IterationNr: 0");
	buildConstraints(constraints, 0,1,3,1,1,1);
	gridbag.setConstraints(itnr, constraints);
	west.add(itnr);

	rad = new JLabel("TrainRadius: 10.00");
	buildConstraints(constraints, 0,2,3,1,1,1);
	gridbag.setConstraints(rad, constraints);
	west.add(rad);

	rat = new JLabel("InputVectorChangeRate: 1.00");
	buildConstraints(constraints, 0,3,3,1,1,1);
	gridbag.setConstraints(rat, constraints);
	west.add(rat);

	mainframe.add(west, BorderLayout.WEST);
	
	setContentPane(mainframe);

	neuron = new Neuron[mNeur][nNeur];
	algo.initKMap(neuron, mNeur, nNeur, invec);
    }
    
    void buildConstraints(GridBagConstraints gbc, int gx, int gy, int gw, int gh, int wx, int wy) {
	gbc.gridx = gx;
	gbc.gridy = gy;
	gbc.gridwidth = gw;
	gbc.gridheight = gh;
	gbc.weightx = wx;
	gbc.weighty = wy;
    }
    
    class InputVectorChoice {
	void plus(){
	    ++invec;
	    inputVec.setText("InputVectors: "+invec);
	    resetMap();
	}
	
	void minus(){
	    if (invec > 1) {
		--invec;
		inputVec.setText("InputVectors: "+invec);
		resetMap();
	    }
	}
    }
    
    class KohonenThreadTrain extends Thread {
	public void run(){
	    while(true) {

		++iteration;
		for (int i=0; i<input.size(); i+=invec) {
		    lineInput = new Vector();
		    for (int j=0; j<invec; j++) {
			lineInput.add(input.get(i+j));
		    }

		    algo.trainMap(neuron, mNeur, nNeur, lineInput, iteration);
		}

		itnr.setText("IterationNr: "+iteration);
		radius = 20/Math.sqrt(iteration);
		rad.setText("TrainRadius: "+Math.floor(radius*1000)/1000);
		rate = 10/Math.sqrt(iteration);
		rat.setText("InputVectorChangeRate: "+Math.floor(rate*1000)/1000);
		repaint();
		if (isInterrupted()==true) { 
		    break;
		}
	    }
	}
    }
    
    class KohonenThreadTest extends Thread {
	public void run(){
	    tested = true;
	    oldStim = new Vector();
	    while(true) {
		for (int i=0; i<input.size(); i +=invec) {
		    lineInput = new Vector();

		    for (int j=0; j<invec; j++) {
			lineInput.add(input.get(i+j));
			stimmax = algo.getStimMax(neuron, mNeur, nNeur, lineInput); 

			if ((oldStim.size()/2)<=input.size()) {
			    oldStim.add((Integer)stimmax.get(1));
			    oldStim.add((Integer)stimmax.get(2));
			}

			repaint();
			try {
			    Thread.sleep(200);
			} catch (InterruptedException e) {
			    interrupt();
			}
		    }
		}

		if (isInterrupted()==true){
		    tested = false;
		    break;
		}
	    }
	}
    }
		
        
    class KohonenFrame extends JPanel {
	int x, y, stx, sty, oldX1, oldX2, oldY1, oldY2;
	
	public void paintComponent(Graphics g){
	    super.paintComponent(g);
	    g.setColor(Color.black);

	    Dimension dx = kFrame.getSize();
	    int maxX = mNeur;
	    int maxY = nNeur;

	    x = (int)dx.getWidth() / (maxX+1);
	    y = (int)dx.getHeight() / (maxY+1);

	    for (int i=0; i< maxX; i++) {
		for (int j=0; j< maxY; j++) {
		    g.fillOval(x*(i+1), y*(j+1),6,6);
		}
	    }
	    if (tested == true) {
		g.setColor(Color.red);
		stx = ((Integer)stimmax.get(1)).intValue();
		sty = ((Integer)stimmax.get(2)).intValue();
		g.fillOval(x*(stx+1), y*(sty+1),6,6);
	
		for (int i=0; i<(oldStim.size()); i +=2){
		    oldX1 =((Integer)oldStim.get(i)).intValue();
		    oldY1 =((Integer)oldStim.get(i+1)).intValue();
		 
		    if (oldStim.size() >= i+3) {
			oldX2 =((Integer)oldStim.get(i+2)).intValue();
			oldY2 =((Integer)oldStim.get(i+3)).intValue();
			g.drawLine(x*(oldX1+1), y*(oldY1+1), x*(oldX2+1),  y*(oldY2+1));
		    }
		}
	    }
	}
    }
    
    public void actionPerformed(ActionEvent evt) {
	Object source = evt.getSource();
	
	if (source == train) {
	    if (train.getText()=="Start Training") {
		try {
		    if (test.getText() == "Stop Test") {
			testing.interrupt();
			test.setText("Start Test");
		    }
		    train.setText("Stop Training");
		    StringTokenizer st = new StringTokenizer(lessons.getText(0,lessons.getLineEndOffset(lessons.getLineCount()-1)));
		    input = new Vector();
		    for (int i=0; i < lessons.getLineCount(); i++){
			for (int j=0; j<invec; j++) {
			    Double d = new Double(st.nextToken());
			    input.add(d);
			}
		    }
		    training = new KohonenThreadTrain();
		    training.start();
		} 
		catch (Exception e) {
		    lessons.insert("Falsche Trainigsformate",0);
		    train.setText("Start Training");
		    repaint();
		}
	    }
	    else if (train.getText() == "Stop Training") {
		training.interrupt();
		train.setText("Start Training");
	    }
	     repaint();
	}
	else if (source == test) {
	    if (test.getText() == "Start Test") {
		try {
		    
		    if (train.getText() == "Stop Training") {
			training.interrupt();
			train.setText("Start Training");
		    }

		    test.setText("Stop Test");
		    StringTokenizer st = new StringTokenizer(lessons.getText(0,lessons.getLineEndOffset(lessons.getLineCount()-1)));
		    input = new Vector();
		    for (int i=0; i < lessons.getLineCount(); i++){
			for (int j=0; j<invec; j++) {
			    Double d = new Double(st.nextToken());
			    input.add(d);
			}
		    }
		    testing = new KohonenThreadTest();
		    testing.start();
		} 
		catch (Exception e) {
		    lessons.insert("Falsche Trainigsformate",0);
		    test.setText("Start Test");
		    repaint();
		}
	    } 
	    else if (test.getText() == "Stop Test") {
		testing.interrupt();
		test.setText("Start Test");
	    }
	}	
	else if (source == succ) {
	    choose.plus();
	    repaint();
	}
	else if (source == pred) {
	    choose.minus();
	    repaint();
	}
	else if (source == reset) {
	    resetMap();
	}

	else if(source==openFile) {
	    String inputValue = JOptionPane.showInputDialog("Open File:"); 
	    
	    if (inputValue!=null) {
		try {
		    URL url = new URL(getCodeBase()+inputValue);
		    URLConnection con = url.openConnection();
		    BufferedReader br = new BufferedReader(new InputStreamReader(con.getInputStream()));
			
		    String str;
		    int line = 0;
		    
		    while ((str = br.readLine()) != null) {
			lessons.append(str);
			lessons.append("\n");
		    }
		}
		catch (MalformedURLException e) {
		    lessons.insert("Fehler beim Laden",0);
		}
		catch (Exception e) {
		    lessons.insert("Fehler!",0);
		}
	    }
	}
    }
    
    public void adjustmentValueChanged(AdjustmentEvent evt) {
	JScrollBar source = (JScrollBar)evt.getAdjustable();

	if (source == nNeurons) {
	    nNeur = source.getValue();
	    nNeuron.setText("Nr. of Neurons  n: "+nNeur+"  ");
	    resetMap();
	    repaint();
	}
	else if (source == mNeurons) {
	    mNeur = source.getValue();
	    mNeuron.setText("Nr. of Neurons m: "+mNeur+"  ");
	    resetMap();
	    repaint();
	}
    }

    public void resetMap() {
	if (test.getText() == "Stop Test") {
	    testing.interrupt();
	    test.setText("Start Test");
	}
	else if (train.getText() == "Stop Training") {
	    training.interrupt();
	    train.setText("Start Training");
	}
	
	input.removeAllElements();
	stimmax.removeAllElements();
	iteration = 0;
	radius = 1.0;
	rate = 1.0;
	itnr.setText("IterationNr: "+iteration);
	radius = 100;
	rad.setText("TrainRadius: 10.00");
	rate = 1.0;
	rat.setText("InputVectorChangeRate: 1.00");
	
	neuron = new Neuron[mNeur][nNeur];
	algo.initKMap(neuron, mNeur, nNeur, invec);
    }
	    
    public void init() {
	Kohonen window = new Kohonen();
	window.setVisible(true);
    }
}

	
    
